﻿using System;
using System.ComponentModel;
using System.Diagnostics.CodeAnalysis;

namespace BackgroundTaskManagerDemo
{
    /// <summary>
    /// Class that manages the execution of background tasks.
    /// </summary>
    public static class BackgroundTaskManager
    {
        /// <summary>
        /// Event invoked when a background task is started.
        /// </summary>
        [SuppressMessage("Microsoft.Usage", "CA2211:NonConstantFieldsShouldNotBeVisible",
            Justification = "Add/remove is thread-safe for events in .NET.")]
        public static EventHandler<EventArgs> BackgroundTaskStarted;

        /// <summary>
        /// Event invoked when a background task completes.
        /// </summary>
        [SuppressMessage("Microsoft.Usage", "CA2211:NonConstantFieldsShouldNotBeVisible",
            Justification = "Add/remove is thread-safe for events in .NET.")]
        public static EventHandler<EventArgs> BackgroundTaskCompleted;

        /// <summary>
        /// Runs a task function on a background thread; invokes a completion action on the main thread.
        /// </summary>
        /// <typeparam name="T">Type of task function result.</typeparam>
        /// <param name="taskFunc">Task function to be run on a background thread.</param>
        /// <param name="completionAction">Completion action to run on the primary thread.</param>
        public static void RunBackgroundTask<T>(Func<T> taskFunc, Action<T> completionAction)
        {
            // Create a BackgroundWorker instance
            var backgroundWorker = new BackgroundWorker();

            // Attach to its DoWork event to run the task function and capture the result
            backgroundWorker.DoWork += delegate(object sender, DoWorkEventArgs e)
            {
                e.Result = taskFunc();
            };

            // Attach to its RunWorkerCompleted event to run the completion action
            backgroundWorker.RunWorkerCompleted += delegate(object sender, RunWorkerCompletedEventArgs e)
            {
                // Invoke the BackgroundTaskCompleted event
                var backgroundTaskFinishedHandler = BackgroundTaskCompleted;
                if (null != backgroundTaskFinishedHandler)
                {
                    backgroundTaskFinishedHandler.Invoke(null, EventArgs.Empty);
                }

                // Call the completion action
                completionAction((T)e.Result);
            };

            // Invoke the BackgroundTaskStarted event
            var backgroundTaskStartedHandler = BackgroundTaskStarted;
            if (null != backgroundTaskStartedHandler)
            {
                backgroundTaskStartedHandler.Invoke(null, EventArgs.Empty);
            }

            // Run the BackgroundWorker asynchronously
            backgroundWorker.RunWorkerAsync();
        }
    }
}
